@@ -36,6 +36,8 @@
 
 static int exit_now=0;
 static unsigned int flags=0;
+static sigset_t signal_mask;
+static int signal_received = 0;
 #define FLAG_NODAEMON	1
 
 static char *parse_cli_args(int argc, char *argv[]);
@@ -44,6 +46,7 @@ static void daemonize(void);
 static void print_start_msg(char *msg);
 static int join_group(int sfd, int loopback, int port);
 static int setup_local_socket(int backlog);
+static inline void process_signals(void);
 
 int main(int argc, char *argv[]){
   int i,error=0;
@@ -176,6 +179,8 @@ int main(int argc, char *argv[]){
 
   while(1){
     int len = addr_size;
+
+    process_signals();
     
     tmp_set = rset;
 
@@ -504,16 +509,28 @@ static int check_cluster_conf(void){
   if(!stat(config_file_location, &stat_buf)){
     doc = xmlParseFile(config_file_location);
     if(!doc){
-      log_err("\nUnable to parse %s.\n"
-	      "You should either:\n"
-	      " 1. Correct the XML mistakes, or\n"
-	      " 2. (Re)move the file and attempt to grab a "
-	      "valid copy from the network.\n", config_file_location);
+      log_err("\nUnable to parse %s.\n", config_file_location);
       return -1;
     }
     xmlFreeDoc(doc);
   } else {
-    /* no cluster.conf file.  This is fine, just need to get it from the network */
+    /*
+     * no cluster.conf file.
+     * We used to simply return 0 here and let go fetch a random
+     * conf file from the network.  This worked well in environments
+     * that had only one cluster in broadcast range, but it has
+     * caused to much confusion when there is more than one cluster.
+     * Additionally, our docs and manuals tell users to copy the
+     * file to each node anyway.
+     *
+     * In the event that there is a cluster.conf file, ccs will
+     * still ensure that it has the most up-to-date copy of the
+     * file in relation to the other nodes in the quorate cluster.
+     */
+    log_err("%s: file not found.\n"
+	    "Please ensure an initial copy of this file is present on all cluster nodes.\n",
+	    config_file_location);
+    return -1;
   }
 
   return 0;
@@ -614,11 +631,16 @@ static void parent_exit_handler(int sig){
  * This handles signals which the daemon might receive.
  */
 static void sig_handler(int sig){
+  sigaddset(&signal_mask, sig);
+  ++signal_received;
+}
+
+static void process_signal(int sig){
   int err;
 
   ENTER("sig_handler");
 
-  switch(sig){
+  switch(sig) {
   case SIGINT:
     log_msg("Stopping ccsd, SIGINT received.\n");
     err = EXIT_SUCCESS;
@@ -631,17 +653,13 @@ static void sig_handler(int sig){
     log_msg("Stopping ccsd, SIGTERM received.\n");
     err = EXIT_SUCCESS;
     break;
-  case SIGSEGV:
-    log_err("Stopping ccsd, SIGSEGV received.\n");
-    err = EXIT_FAILURE;
-    break;
   case SIGHUP:
     log_msg("SIGHUP received.\n");
     log_msg("Use ccs_tool for updates.\n");
     return;
     break;
   default:
-    log_err("Stopping ccsd, unknown signal received.\n");
+    log_err("Stopping ccsd, unknown signal %d received.\n", sig);
     err = EXIT_FAILURE;
   }
 
@@ -650,6 +668,24 @@ static void sig_handler(int sig){
 }
 
 
+static inline void process_signals(void)
+{
+  int x;
+
+  if (!signal_received)
+    return;
+
+  signal_received = 0;
+
+  for (x = 1; x < _NSIG; x++) {
+    if (sigismember(&signal_mask, x)) {
+      sigdelset(&signal_mask, x);
+      process_signal(x);
+    }
+  }
+}
+
+
 /**
  * daemonize
  *
@@ -671,11 +707,6 @@ static void daemonize(void){
     if((error = create_lockfile(lockfile_location))){
       goto fail;
     }
-    signal(SIGINT, &sig_handler);
-    signal(SIGQUIT, &sig_handler);
-    signal(SIGTERM, &sig_handler);
-    signal(SIGSEGV, &sig_handler);
-    signal(SIGHUP, &sig_handler);
   } else {
     log_dbg("Entering daemon mode.\n");
 
@@ -729,13 +760,16 @@ static void daemonize(void){
     /* hold off on shutting down parent.  Let cluster_communicator do it **
     ** after if figures out if it can use magma or not.................. **
     **    kill(getppid(), SIGTERM); */
-    signal(SIGINT, &sig_handler);
-    signal(SIGQUIT, &sig_handler);
-    signal(SIGTERM, &sig_handler);
-    signal(SIGSEGV, &sig_handler);
-    signal(SIGHUP, &sig_handler);
   }
 
+  signal(SIGINT, &sig_handler);
+  signal(SIGQUIT, &sig_handler);
+  signal(SIGTERM, &sig_handler);
+  signal(SIGHUP, &sig_handler);
+  signal(SIGPIPE, SIG_IGN);
+  sigemptyset(&signal_mask);
+  signal_received = 0;
+
  fail:
   EXIT("daemonize");
 
