@@ -1233,9 +1233,13 @@ int process_request(int afd){
  fail:
   error = write(afd, ch, sizeof(comm_header_t)+ch->comm_payload_size);
   if(error < 0){
-    if (errno == EINTR)
+  if (errno == EINTR)
       goto fail;
-    log_sys_err("Unable to write package back to sender");
+    if (errno == EPIPE) {
+      error = 0;
+    } else {
+      log_sys_err("Unable to write package back to sender");
+    }
   } else if(error < (sizeof(comm_header_t)+ch->comm_payload_size)){
     log_err("Unable to write complete package.\n");
     error = -EBADE;
@@ -1246,7 +1250,6 @@ int process_request(int afd){
 
   if(ch){ free(ch); }
   if(payload){ free(payload); }
-  
 
   EXIT("process_request");
   return error;
