@@ -419,9 +419,12 @@ static int membership_kthread(void *unused)
 		if (skb_peek(&mem_socket->sk->sk_receive_queue)) {
 			if (dispatch_messages(mem_socket) < 0)
 				goto leave_cluster;
-
 		}
 
+		/* Messages may cause us to quit */
+		if (quit_threads)
+			goto leave_cluster;
+
 		/* Check this again here, in case nodes die while we're doing stuff */
 		if (test_and_clear_bit(WAKE_FLAG_DEADNODE, &wake_flags)) {
 			process_dead_nodes();
@@ -1065,6 +1068,12 @@ int send_kill(int nodeid, int needack)
 {
 	char killmsg;
 	struct sockaddr_cl saddr;
+	int flags;
+
+	if (needack)
+		flags = MSG_QUEUE;
+	else
+		flags = MSG_NOACK;
 
 	killmsg = CLUSTER_MEM_KILL;
 
@@ -1072,7 +1081,7 @@ int send_kill(int nodeid, int needack)
 	saddr.scl_port = CLUSTER_PORT_MEMBERSHIP;
 	saddr.scl_nodeid = nodeid;
 	return kcl_sendmsg(mem_socket, &killmsg, 1, &saddr,
-			   sizeof (struct sockaddr_cl), needack?0:MSG_NOACK);
+			   sizeof (struct sockaddr_cl), flags);
 }
 
 /* Tell the rest of the cluster a node has gone down */
@@ -1361,7 +1370,7 @@ void a_node_just_died(struct cluster_node *node)
 	 * list and wake it */
 	if (current != membership_task) {
 		struct cl_new_dead_node *newnode =
-		    kmalloc(sizeof (struct cl_new_dead_node), GFP_KERNEL);
+			kmalloc(sizeof (struct cl_new_dead_node), GFP_KERNEL);
 		if (!newnode)
 			return;
 		newnode->node = node;
@@ -1369,7 +1378,10 @@ void a_node_just_died(struct cluster_node *node)
 		list_add_tail(&newnode->list, &new_dead_node_list);
 		set_bit(WAKE_FLAG_DEADNODE, &wake_flags);
 		spin_unlock(&new_dead_node_lock);
-		wake_up_process(membership_task);
+		spin_lock(&membership_task_lock);
+		if (membership_task)
+			wake_up_process(membership_task);
+		spin_unlock(&membership_task_lock);
 		P_MEMB("Passing dead node %s onto kmembershipd\n", node->name);
 		return;
 	}
@@ -1382,6 +1394,7 @@ void a_node_just_died(struct cluster_node *node)
 	if (node->state == NODESTATE_MEMBER)
 		cluster_members--;
 	node->state = NODESTATE_DEAD;
+	node->last_seq_recv = 0;
 	up(&cluster_members_lock);
 
 	send_nodedown(node->node_id, node->leave_reason);
@@ -1870,6 +1883,7 @@ static int do_process_startack(struct msghdr *msg, char *buf, int len)
 				leavereason = 0;
 				joining_temp_nodeid = 0;
 				node_state = MEMBER;
+				notify_listeners();
 				sm_member_update(cluster_is_quorate);
 			}
 		}
@@ -2877,6 +2891,13 @@ static int do_process_hello(struct msghdr *msg, char *buf, int len)
 		}
 	}
 
+	/* If we get a master hello and we are not the master then start a CHECK transition, cos the
+	   real master must have gone away in a period of confusion */
+	if (node_state != MASTER && hellomsg->flags & HELLO_FLAG_MASTER) {
+		node = find_node_by_nodeid(saddr->scl_nodeid);
+		start_transition(TRANS_CHECK, node);
+	}
+
 	return 0;
 
 }
@@ -2957,7 +2978,7 @@ struct cluster_node *find_node_by_nodeid(unsigned int id)
 {
 	struct cluster_node *node;
 
-	if (id > sizeof_members_array)
+	if (id >= sizeof_members_array)
 		return NULL;
 
 	spin_lock(&members_by_nodeid_lock);
@@ -3141,8 +3162,7 @@ static int elect_master(struct cluster_node **master_node, int disallow_node)
 
 	for (i = 1; i < sizeof_members_array; i++) {
 		if (members_by_nodeid[i] &&
-		    members_by_nodeid[i]->state == NODESTATE_MEMBER &&
-		    i != disallow_node) {
+		    members_by_nodeid[i]->state == NODESTATE_MEMBER) {
 			*master_node = members_by_nodeid[i];
 			P_MEMB("Elected master is %s\n", (*master_node)->name);
 			return (*master_node)->us;
