@@ -247,6 +247,8 @@ static int check_join_notice(sm_sevent_t *sev)
 	} else if (!pos && !wait && !restart && neg) {
 		/* we're the first in the cluster to join this sg */
 		sev->se_sg->global_id = sm_new_global_id(sev->se_sg->level);
+		if (sev->se_sg->global_id == 0)
+			error = -EOVERFLOW;
 	} else
 		error = -1;
 
