@@ -41,15 +41,17 @@ void init_messages(void)
 
 uint32_t sm_new_global_id(int level)
 {
-	uint32_t id = global_last_id++;
+	uint32_t id = global_last_id + sm_our_nodeid;
 	uint8_t l = (uint8_t) level;
 
 	if (level > 255)
 		return 0;
 
-	if (id > 0x00FFFFFF)
+	if (id > 0x00FFFFFF) {
+		log_print("ERROR: max global group id");
 		return 0;
-
+	}
+	global_last_id = id;
 	id |= (l << 24);
 	return id;
 }
