@@ -31,7 +31,7 @@ static uint32_t lookup_bcast(uint32_t localaddr, char *ifname)
 
     ifname[0] = '\0';
 
-    for (iindex = 0; iindex < 16; iindex++) {
+    for (iindex = 0; iindex < 255; iindex++) {
 	ifr.ifr_ifindex = iindex;
 	if (ioctl(sock, SIOCGIFNAME, &ifr) == 0) {
 	    ifr.ifr_ifindex = iindex;
@@ -77,6 +77,7 @@ static int setup_ipv4_interface(commandline_t *comline, int num, struct hostent
     int mcast_sock;
     int local_sock;
     int ret;
+    int one=1;
     int he_errno;
     uint32_t bcast;
 
@@ -138,7 +139,6 @@ static int setup_ipv4_interface(commandline_t *comline, int num, struct hostent
     }
     else {
 	/* Broadcast */
-	int one=1;
 	memcpy(&mcast_sin.sin_addr, &bcast, sizeof(struct in_addr));
 	if (setsockopt(mcast_sock, SOL_SOCKET, SO_BROADCAST, (void *)&one, sizeof(int)))
 	    die("Can't enable broadcast: %s", strerror(errno));
@@ -174,6 +174,9 @@ static int setup_ipv4_interface(commandline_t *comline, int num, struct hostent
     if (local_sock < 0)
 	die("Can't open local socket: %s", strerror(errno));
 
+    if (setsockopt(local_sock, SOL_SOCKET, SO_BROADCAST, (void *)&one, sizeof(int)))
+        die("Can't enable broadcast: %s", strerror(errno));
+    
     if (bind(local_sock, (struct sockaddr *)&local_sin, sizeof(local_sin)))
 	die("Cannot bind local address: %s", strerror(errno));
 
@@ -360,7 +363,10 @@ int join(commandline_t *comline)
         die("Node is already active");
 
     /* Set the node name - without domain part */
-    strcpy(nodename, comline->nodenames[0]);
+    if (comline->override_nodename)
+	strcpy(nodename, comline->override_nodename);
+    else
+    	strcpy(nodename, comline->nodenames[0]);
 
     error = ioctl(cluster_sock, SIOCCLUSTER_SET_NODENAME, nodename);
     if (error)
@@ -374,6 +380,13 @@ int join(commandline_t *comline)
 	    die("Unable to set cluster nodeid: %s", cman_error(errno));
     }
 
+    if (comline->cluster_id) {
+	error = ioctl(cluster_sock, SIOCCLUSTER_SET_CLUSTERID,
+		      comline->cluster_id);
+	if (error)
+	    die("Unable to set cluster_id: %s", cman_error(errno));
+    }
+
     /*
      * Setup the interface/multicast
      */
