@@ -28,6 +28,7 @@
 #define CONFIG_VERSION_PATH	"/cluster/@config_version"
 #define EXP_VOTES_PATH		"/cluster/cman/@expected_votes"
 #define TWO_NODE_PATH		"/cluster/cman/@two_node"
+#define CLUSTER_ID_PATH		"/cluster/cman/@cluster_id"
 #define MCAST_ADDR_PATH		"/cluster/cman/multicast/@addr"
 #define PORT_PATH		"/cluster/cman/@port"
 #define CMAN_PREFIX		"/cluster/cman/@"
@@ -524,6 +525,13 @@ int get_ccs_join_info(commandline_t *comline)
 		}
 	}
 
+	/* cluster_id override? */
+	error = ccs_get(cd, CLUSTER_ID_PATH, &str);
+	if (!error) {
+		comline->cluster_id = atoi(str);
+		free(str);
+	}
+
 	get_ccs_cman_config(cd, comline);
 
 	ccs_disconnect(cd);
