@@ -1,7 +1,7 @@
 /******************************************************************************
 *******************************************************************************
 **
-**  Copyright (C) 2004 Red Hat, Inc.  All rights reserved.
+**  Copyright (C) 2004-2006 Red Hat, Inc.  All rights reserved.
 **
 **  This library is free software; you can redistribute it and/or
 **  modify it under the terms of the GNU Lesser General Public
@@ -163,6 +163,11 @@ int cman_dispatch(cman_handle_t handle, int flags)
 		if (len < 0 && errno == EAGAIN)
 			return len;
 
+		if (len == 0) {
+			errno = EHOSTDOWN;
+			return -1;
+		}
+
 		/* Send a callback if registered */
 		if (msg.msg_flags & MSG_OOB)
 		{
@@ -196,7 +201,7 @@ int cman_dispatch(cman_handle_t handle, int flags)
 
 	}
 	while ( flags & CMAN_DISPATCH_ALL &&
-		(len < 0 && errno == EAGAIN) );
+		!(len < 0 && errno == EAGAIN) );
 
 	return len;
 }
@@ -468,3 +473,35 @@ int cman_poll_quorum_device(cman_handle_t handle, int isavailable)
 	struct cman_handle *h = (struct cman_handle *)handle;
 	return ioctl(h->fd, SIOCCLUSTER_QD_POLL, isavailable, sizeof(int));
 }
+
+int cman_get_subsys_count(cman_handle_t handle)
+{
+	struct cman_handle *h = (struct cman_handle *)handle;
+	return ioctl(h->fd, SIOCCLUSTER_GET_JOINCOUNT, NULL, 0);
+}
+
+
+int cman_kill_node(cman_handle_t handle, int nodeid)
+{
+	struct cman_handle *h = (struct cman_handle *)handle;
+	return ioctl(h->fd, SIOCCLUSTER_KILLNODE, nodeid, sizeof(int));
+}
+
+
+int cman_set_expected_votes(cman_handle_t handle, int evotes)
+{
+	struct cman_handle *h = (struct cman_handle *)handle;
+	return ioctl(h->fd, SIOCCLUSTER_SETEXPECTED_VOTES, evotes, sizeof(int));
+}
+
+
+int cman_set_votes(cman_handle_t handle, int votes, int nodeid)
+{
+	struct cman_handle *h = (struct cman_handle *)handle;
+	return ioctl(h->fd, SIOCCLUSTER_SET_VOTES, nodeid, sizeof(int));
+}
+
+int cman_replyto_shutdown(cman_handle_t handle, int yesno)
+{
+	return 0;
+}
