@@ -1,7 +1,7 @@
 /******************************************************************************
 *******************************************************************************
 **
-**  Copyright (C) 2004 Red Hat, Inc.  All rights reserved.
+**  Copyright (C) 2004-2006 Red Hat, Inc.  All rights reserved.
 **
 **  This library is free software; you can redistribute it and/or
 **  modify it under the terms of the GNU Lesser General Public
@@ -23,6 +23,7 @@
 #ifndef _LIBCMAN_H_
 #define _LIBCMAN_H_
 
+#include <netinet/in.h>
 #define CMAN_MAX_ADDR_LEN sizeof(struct sockaddr_in6)
 #define CMAN_MAX_NODENAME_LEN 255
 #define MAX_CLUSTER_NAME_LEN   16
@@ -80,22 +81,22 @@ typedef struct cman_cluster
 } cman_cluster_t;
 
 /* Callback routine for a membership event */
-typedef void (*cman_callback_t)(cman_handle_t handle, void *private, int reason, int arg);
+typedef void (*cman_callback_t)(cman_handle_t handle, void *privdata, int reason, int arg);
 
 /* Callback routine for data received */
-typedef void (*cman_datacallback_t)(cman_handle_t handle, void *private,
+typedef void (*cman_datacallback_t)(cman_handle_t handle, void *privdata,
 				    char *buf, int len, uint8_t port, int nodeid);
 
 
 /* cman_init    returns the handle you need to pass to the other API calls,
    cman_finish  destroys that handle
 */
-cman_handle_t cman_init(void *private);
+cman_handle_t cman_init(void *privdata);
 int cman_finish(cman_handle_t handle);
 
 /* Update/retrieve private data */
-int cman_set_private(cman_handle_t *h, void *private);
-int cman_get_private(cman_handle_t *h, void **private);
+int cman_set_private(cman_handle_t *h, void *privdata);
+int cman_get_private(cman_handle_t *h, void **privdata);
 
 /* Notification of membership change events. NOte that these are sent after
    a transition so multiple nodes may have left the cluster (but a maximum of
@@ -119,6 +120,15 @@ int cman_is_listening(cman_handle_t handle, int nodeid, uint8_t port);
 int cman_is_quorate(cman_handle_t handle);
 int cman_get_version(cman_handle_t handle, cman_version_t *version);
 int cman_get_cluster(cman_handle_t handle, cman_cluster_t *clinfo);
+int cman_get_subsys_count(cman_handle_t handle);
+
+
+/* "You should know what you're doing" calls */
+int cman_kill_node(cman_handle_t handle, int nodeid);
+int cman_set_expected_votes(cman_handle_t handle, int evotes);
+
+/* Warning, 'nodeid' is ignored in this version of the library */
+int cman_set_votes(cman_handle_t handle, int votes, int nodeid);
 
 
 
@@ -152,4 +162,5 @@ int cman_register_quorum_device(cman_handle_t handle, char *name, int votes);
 int cman_unregister_quorum_device(cman_handle_t handle);
 int cman_poll_quorum_device(cman_handle_t handle, int isavailable);
 
+int cman_replyto_shutdown(cman_handle_t handle, int yesno);
 #endif
