@@ -344,7 +344,7 @@ static void ast_routine(void *param)
 
 			/* If the LKB has been freed then we need to tidy up too */
 			lkb = dlm_get_lkb(li->li_file->fi_ls->ls_lockspace, li->li_lksb.sb_lkid);
-			if (!lkb) {
+			if (!lkb && test_bit(LI_FLAG_ONLIST, &li->li_flags)) {
 				spin_lock(&li->li_file->fi_lkb_lock);
 				list_del(&li->li_ownerqueue);
 				clear_bit(LI_FLAG_ONLIST, &li->li_flags);
@@ -486,7 +486,7 @@ static int dlm_close(struct inode *inode, struct file *file)
 	 * because the lkbs are freed during dlm_unlock operations
 	 */
 	list_for_each_entry_safe(old_li, safe, &f->fi_lkb_list, li_ownerqueue) {
-		int status;
+		int status = 0;
 		int lock_status;
 		int flags = 0;
 		struct dlm_lkb *lkb;
@@ -520,36 +520,40 @@ static int dlm_close(struct inode *inode, struct file *file)
 
 		clear_bit(LI_FLAG_COMPLETE, &li.li_flags);
 
-		/* If it's not granted then cancel the request.
-		 * If the lock was WAITING then it will be dropped,
-		 *    if it was converting then it will be reverted to GRANTED,
-		 *    then we will unlock it.
-		 */
-		lock_status = lkb->lkb_status;
+		/* If the LKB has gone, then there's nothing to unlock */
+		if (lkb) {
+			/* If it's not granted then cancel the request.
+			 * If the lock was WAITING then it will be dropped,
+			 *    if it was converting then it will be reverted to GRANTED,
+			 *    then we will unlock it.
+			 */
+			lock_status = lkb->lkb_status;
 
-		if (lock_status != GDLM_LKSTS_GRANTED)
-			flags = DLM_LKF_CANCEL;
+			if (lock_status != GDLM_LKSTS_GRANTED)
+				flags = DLM_LKF_CANCEL;
 
-		if (lkb->lkb_grmode >= DLM_LOCK_PW)
-			flags |= DLM_LKF_IVVALBLK;
+			if (lkb->lkb_grmode >= DLM_LOCK_PW)
+				flags |= DLM_LKF_IVVALBLK;
 
-		status = dlm_unlock(f->fi_ls->ls_lockspace, lkb->lkb_id, flags, &li.li_lksb, &li);
-
-		/* Must wait for it to complete as the next lock could be its
-		 * parent */
-		if (status == 0)
-			wait_for_ast(&li);
-
-		/* If it was waiting for a conversion, it will
-		   now be granted so we can unlock it properly */
-		if (lock_status == GDLM_LKSTS_CONVERT) {
-			flags &= ~DLM_LKF_CANCEL;
-			clear_bit(LI_FLAG_COMPLETE, &li.li_flags);
 			status = dlm_unlock(f->fi_ls->ls_lockspace, lkb->lkb_id, flags, &li.li_lksb, &li);
 
+			/* Must wait for it to complete as the next lock could be its
+			 * parent */
 			if (status == 0)
 				wait_for_ast(&li);
+
+			/* If it was waiting for a conversion, it will
+			   now be granted so we can unlock it properly */
+			if (lock_status == GDLM_LKSTS_CONVERT) {
+				flags &= ~DLM_LKF_CANCEL;
+				clear_bit(LI_FLAG_COMPLETE, &li.li_flags);
+				status = dlm_unlock(f->fi_ls->ls_lockspace, lkb->lkb_id, flags, &li.li_lksb, &li);
+
+				if (status == 0)
+					wait_for_ast(&li);
+			}
 		}
+
 		/* Unlock suceeded, free the lock_info struct. */
 		if (status == 0) {
 			kfree(old_li);
