@@ -265,6 +265,7 @@ static int new_lockspace(char *name, int namelen, void **lockspace, int flags)
 
 	ls = find_lockspace_by_name(name, namelen);
 	if (ls) {
+		log_error(ls, "lockspace already in use");
 		*lockspace = (void *)(long) ls->ls_local_id;
 		module_put(THIS_MODULE);
 		return -EEXIST;
@@ -321,6 +322,7 @@ static int new_lockspace(char *name, int namelen, void **lockspace, int flags)
 
 	INIT_LIST_HEAD(&ls->ls_nodes);
 	INIT_LIST_HEAD(&ls->ls_nodes_gone);
+	spin_lock_init(&ls->ls_nodes_gone_spin);
 	ls->ls_num_nodes = 0;
 	ls->ls_node_array = NULL;
 	ls->ls_recoverd_task = NULL;
@@ -600,6 +602,8 @@ void dlm_emergency_shutdown()
 	struct dlm_ls *ls;
 	struct dlm_ls *tmp;
 
+	printk("WARNING: dlm_emergency_shutdown\n");
+
 	/* Shut lowcomms down to prevent any socket activity */
 	lowcomms_stop_accept();
 
@@ -617,6 +621,8 @@ void dlm_emergency_shutdown()
 	}
 
 	spin_unlock(&lslist_lock);
+
+	printk("WARNING: dlm_emergency_shutdown\n");
 }
 
 struct dlm_recover *allocate_dlm_recover(void)
