@@ -133,7 +133,7 @@ static wait_queue_head_t lowcomms_recv_waitq;
 /* An array of pointers to connections, indexed by NODEID */
 static struct connection **connections;
 static struct semaphore connections_lock;
-static kmem_cache_t *con_cache;
+static struct kmem_cache *con_cache;
 static int conn_array_size;
 static atomic_t accepting;
 
