@@ -62,7 +62,7 @@ int debug;
 int operation;
 int child_wait;
 int quorum_wait = TRUE;
-int fenced_start_timeout = 0;
+int fenced_timeout = 0;
 int signalled = 0;
 int cl_sock;
 char our_name[MAX_CLUSTER_MEMBER_NAME_LEN+1];
@@ -282,9 +282,9 @@ static int do_join(int argc, char *argv[])
 
 	setup_sock();
 
-       	if (fenced_start_timeout) {
+       	if (fenced_timeout) {
                	signal(SIGALRM, sigalarm_handler);
-               	alarm(fenced_start_timeout);
+               	alarm(fenced_timeout);
        	}
 
 	if (!check_quorum()) {
@@ -373,8 +373,17 @@ static int do_leave(void)
 	check_mounted();
 	setup_sock();
 
-	if (!check_quorum())
+	if (fenced_timeout) {
+               	signal(SIGALRM, sigalarm_handler);
+               	alarm(fenced_timeout);
+       	}
+
+	if (!check_quorum()) {
+		if (errno == ETIMEDOUT)
+			printf("%s: Timed out waiting for cluster "
+			       "quorum to form.\n", prog_name);
 		return EXIT_FAILURE;
+	}
 
 	close(cl_sock);
 
@@ -487,7 +496,7 @@ static void decode_arguments(int argc, char *argv[])
 			break;
 
                 case 't':
-                        fenced_start_timeout = get_int_arg(optchar, optarg);
+                        fenced_timeout = get_int_arg(optchar, optarg);
                         break;
 
 		case 'c':
