@@ -23,7 +23,7 @@ static int usr_interrupt;
 char our_name[MAX_CLUSTER_MEMBER_NAME_LEN+1];
 
 
-#define OPTION_STRING			("cj:f:t:Dn:hVSwQ")
+#define OPTION_STRING			("cj:f:t:Dn:O:hVSwQ")
 #define LOCKFILE_NAME			"/var/run/fenced.pid"
 
 
@@ -40,6 +40,8 @@ static void print_usage(void)
 			           DEFAULT_POST_JOIN_DELAY);
 	printf("  -f <secs>        Post-fail fencing delay (default %d)\n",
 				   DEFAULT_POST_FAIL_DELAY);
+ 	printf("  -O <path>        Override path (default %s)\n",
+ 	       			   DEFAULT_OVERRIDE_PATH);
 	printf("  -D               Enable debugging code and don't fork\n");
 	printf("  -h               Print this help, then exit\n");
 	printf("  -n <name>        Name of the fence domain, \"default\" if none\n");
@@ -434,6 +436,23 @@ static int check_ccs(fd_t *fd)
 			free(str);
 	}
 
+	if (fd->comline->override_path_opt == FALSE) {
+		str = NULL;
+		memset(path, 0, 256);
+		sprintf(path, "/cluster/fence_daemon/@override_path");
+
+		error = ccs_get(cd, path, &str);
+		if (!error)
+			/* XXX These are not explicitly freed on exit; if
+			   we decide to make fenced handle SIGHUP at a later
+			   time, we will need to free this. */
+			fd->comline->override_path = strdup(str);
+		else
+			fd->comline->override_path = strdup(DEFAULT_OVERRIDE_PATH);
+		if (str)
+			free(str);
+	}
+
 	log_debug("delay post_join %ds post_fail %ds",
 		  fd->comline->post_join_delay, fd->comline->post_fail_delay);
 
@@ -527,6 +546,8 @@ static void decode_arguments(int argc, char **argv, commandline_t *comline)
 	int cont = TRUE;
 	int optchar;
 
+	comline->override_path_opt = FALSE;
+	comline->override_path = NULL;
 	comline->post_join_delay_opt = FALSE;
 	comline->post_fail_delay_opt = FALSE;
 	comline->clean_start_opt = FALSE;
@@ -551,6 +572,11 @@ static void decode_arguments(int argc, char **argv, commandline_t *comline)
 			comline->post_fail_delay_opt = TRUE;
 			break;
 
+		case 'O':
+			comline->override_path = strdup(optarg);
+			comline->override_path_opt = TRUE;
+			break;
+
 		case 'D':
 			comline->debug = TRUE;
 			fenced_debug = TRUE;
