@@ -24,10 +24,11 @@
 #include <linux/socket.h>
 #include <linux/kthread.h>
 #include <net/sock.h>
-#include <linux/lm_interface.h>
-#include <cluster/cnxman.h>
-#include <cluster/service.h>
-#include <cluster/dlm.h>
+
+#include "lm_interface.h"
+#include "cluster/cnxman.h"
+#include "cluster/service.h"
+#include "cluster/dlm.h"
 
 /* We take a shortcut and use lm_lockname structs for internal locks.  This
    means we must be careful to keep these types different from those used in
@@ -46,7 +47,7 @@
 
 #define LOCK_DLM_MAX_NODES	(128)
 
-#define DROP_LOCKS_COUNT	(50000)
+#define DROP_LOCKS_COUNT	(200000)
 #define DROP_LOCKS_PERIOD	(60)
 #define SHRINK_CACHE_COUNT	(100)
 #define SHRINK_CACHE_MAX	(1000)
@@ -146,6 +147,7 @@ struct posix_lock {
 	struct dlm_resource *   resource;
 	dlm_lock_t *            lp;
 	unsigned long           owner;
+	unsigned int            pid;
 	uint64_t                start;
 	uint64_t                end;
 	int                     count;
