@@ -50,7 +50,7 @@ static int diaper_major = 0;
 static LIST_HEAD(diaper_list);
 static spinlock_t diaper_lock;
 static DEFINE_IDR(diaper_idr);
-kmem_cache_t *diaper_slab;
+struct kmem_cache *diaper_slab;
 
 /**
  * diaper_open -
@@ -232,9 +232,9 @@ get_dummy_sb(struct diaper_holder *dh)
 	struct inode *inode;
 	int error;
 
-	mutex_lock(&real->bd_mount_mutex);
+	down(&real->bd_mount_sem);
 	sb = sget(&gfs_fs_type, gfs_test_bdev_super, gfs_set_bdev_super, real);
-	mutex_unlock(&real->bd_mount_mutex);
+	up(&real->bd_mount_sem);
 	if (IS_ERR(sb))
 		return PTR_ERR(sb);
 
@@ -433,6 +433,7 @@ diaper_put(struct diaper_holder *dh)
 	struct gendisk *gd = dh->dh_gendisk;
 	int minor = dh->dh_gendisk->first_minor;
 
+	down_write(&dh->dh_dummy_sb->s_umount);
 	generic_shutdown_super(dh->dh_dummy_sb);
 
 	mempool_destroy(dh->dh_mempool);
