@@ -59,6 +59,7 @@
 
 #include <linux/sched.h>
 #include <linux/slab.h>
+#include <linux/vmalloc.h>
 #include <linux/smp_lock.h>
 #include <linux/spinlock.h>
 #include <asm/semaphore.h>
@@ -707,7 +708,7 @@ dir_split_leaf(struct gfs_inode *dip, uint32_t index, uint64_t leaf_no)
 	uint64_t bn, *lp;
 	uint32_t name_len;
 	int x, moved = FALSE;
-	int error;
+	int error, lp_vfree=0;
 
 	/*  Allocate the new leaf block  */
 
@@ -756,7 +757,16 @@ dir_split_leaf(struct gfs_inode *dip, uint32_t index, uint64_t leaf_no)
 	   Don't bother distinguishing stuffed from non-stuffed.
 	   This code is complicated enough already. */
 
-	lp = gmalloc(half_len * sizeof(uint64_t));
+	lp = kmalloc(half_len * sizeof (uint64_t), GFP_KERNEL);
+	if (unlikely(!lp)) {
+		lp = vmalloc(half_len * sizeof (uint64_t));
+		if (!lp) {
+			printk("GFS: dir_split_leaf vmalloc fail - half_len=%d\n", half_len);
+			error = -ENOMEM;
+			goto fail_brelse;
+		} else
+			lp_vfree = 1;
+	}
 
 	error = gfs_internal_read(dip, (char *)lp, start * sizeof(uint64_t),
 				  half_len * sizeof(uint64_t));
@@ -779,7 +789,10 @@ dir_split_leaf(struct gfs_inode *dip, uint32_t index, uint64_t leaf_no)
 		goto fail_lpfree;
 	}
 
-	kfree(lp);
+	if (unlikely(lp_vfree))
+		vfree(lp);
+	else
+		kfree(lp);
 
 	/*  Compute the divider  */
 
@@ -852,7 +865,10 @@ dir_split_leaf(struct gfs_inode *dip, uint32_t index, uint64_t leaf_no)
 	return error;
 
  fail_lpfree:
-	kfree(lp);
+	if (unlikely(lp_vfree))
+		vfree(lp);
+	else
+		kfree(lp);
 
  fail_brelse:
 	brelse(obh);
@@ -1085,14 +1101,21 @@ do_filldir_single(struct gfs_inode *dip, uint64_t *offset,
 	struct gfs_dirent **darr;
 	struct gfs_dirent *de;
 	unsigned int e = 0;
-	int error;
+	int error, do_vfree=0;
 
 	if (!entries)
 		return 0;
 
 	darr = kmalloc(entries * sizeof(struct gfs_dirent *), GFP_KERNEL);
-	if (!darr)
-		return -ENOMEM;
+	if (unlikely(!darr)) {
+		darr = vmalloc(entries * sizeof (struct gfs_dirent *));
+		if (!darr) {
+			printk("GFS: do_filldir_single vmalloc fails, entries=%d\n", entries);
+			return -ENOMEM;
+		}
+	else
+		do_vfree = 1;
+	}
 
 	dirent_first(dip, bh, &de);
 	do {
@@ -1117,7 +1140,10 @@ do_filldir_single(struct gfs_inode *dip, uint64_t *offset,
 				entries, copied);
 
  out:
-	kfree(darr);
+	if (unlikely(do_vfree))
+		vfree(darr);
+	else
+		kfree(darr);
 
 	return error;
 }
@@ -1148,7 +1174,7 @@ do_filldir_multi(struct gfs_inode *dip, uint64_t *offset,
 	unsigned int leaves = 0, l = 0;
 	unsigned int x;
 	uint64_t ln;
-	int error = 0;
+	int error = 0, leaves_vfree=0, entries_vfree=0;
 
 	/*  Count leaves and entries  */
 
@@ -1182,11 +1208,31 @@ do_filldir_multi(struct gfs_inode *dip, uint64_t *offset,
 
 	if (leaves) {
 		larr = kmalloc(leaves * sizeof(struct buffer_head *), GFP_KERNEL);
-		if (!larr)
-			return -ENOMEM;
+		if (unlikely(!larr)) {
+			larr = vmalloc(leaves * sizeof (struct buffer_head *));
+			if (!larr) {
+				printk("GFS: do_filldir_multi vmalloc fails leaves=%d\n", leaves);
+				return -ENOMEM;
+			} else
+				leaves_vfree = 1;
+		}
 	}
 
 	darr = kmalloc(entries * sizeof(struct gfs_dirent *), GFP_KERNEL);
+	if (unlikely(!darr)) {
+		darr = vmalloc(entries * sizeof (struct gfs_dirent *));
+		if (!darr) {
+			printk("GFS: do_filldir_multi vmalloc fails entries=%d\n", entries);
+			if (larr) {
+				if (leaves_vfree) 
+					vfree(larr);
+				else 
+					kfree(larr);
+			}
+			return -ENOMEM;
+		} else
+			entries_vfree = 1;
+	} 
 	if (!darr) {
 		if (larr)
 			kfree(larr);
@@ -1261,13 +1307,20 @@ do_filldir_multi(struct gfs_inode *dip, uint64_t *offset,
 	/*  Clean up  */
 
  out:
-	kfree(darr);
+	if (unlikely(entries_vfree))
+		vfree(darr);
+	else
+		kfree(darr);
 
 	for (x = 0; x < l; x++)
 		brelse(larr[x]);
 
-	if (leaves)
-		kfree(larr);
+	if (leaves) {
+		if (unlikely(leaves_vfree))
+			vfree(larr);
+		else
+			kfree(larr);
+	}
 
 	return error;
 }
@@ -2094,17 +2147,12 @@ leaf_free(struct gfs_inode *dip,
 	struct buffer_head *bh, *dibh;
 	uint64_t blk;
 	unsigned int rg_blocks = 0;
-	char *ht;
+	char *ht=0;
 	unsigned int x, size = len * sizeof(uint64_t);
 	int error;
 
 	memset(&rlist, 0, sizeof(struct gfs_rgrp_list));
 
-	ht = kmalloc(size, GFP_KERNEL);
-	if (!ht)
-		return -ENOMEM;
-	memset(ht, 0, size);
-
 	gfs_alloc_get(dip);
 
 	error = gfs_quota_hold_m(dip, NO_QUOTA_CHANGE, NO_QUOTA_CHANGE);
@@ -2165,7 +2213,8 @@ leaf_free(struct gfs_inode *dip,
 		dip->i_di.di_blocks--;
 	}
 
-	error = gfs_internal_write(dip, ht, index * sizeof(uint64_t), size);
+	error = gfs_writei(dip, ht, index * sizeof (uint64_t), size, gfs_zero_blocks, NULL);
+
 	if (error != size) {
 		if (error >= 0)
 			error = -EIO;
@@ -2195,7 +2244,6 @@ leaf_free(struct gfs_inode *dip,
 
  out:
 	gfs_alloc_put(dip);
-	kfree(ht);
 
 	return error;
 }
