@@ -345,6 +345,8 @@ gi_get_counters(struct gfs_inode *ip,
 		   atomic_read(&sdp->sd_glock_count));
 	gfs_printf("sd_glock_held_count:locks held::%d\n",
 		   atomic_read(&sdp->sd_glock_held_count));
+	gfs_printf("sd_freeze_count:freeze count::%d\n",
+		   sdp->sd_freeze_count);
 	gfs_printf("sd_inode_count:incore inodes::%d\n",
 		   atomic_read(&sdp->sd_inode_count));
 	gfs_printf("sd_bufdata_count:metadata buffers::%d\n",
@@ -1229,7 +1231,7 @@ gi_do_hfile_write(struct gfs_sbd *sdp, struct gfs_ioctl *gi)
 	}
 
 	error = gfs_writei(ip, gi->gi_data, gi->gi_offset, gi->gi_size,
-			   gfs_copy_from_user);
+					   gfs_copy_from_user, NULL);
 
 	gfs_trans_end(sdp);
 
