@@ -2,7 +2,7 @@
 *******************************************************************************
 **
 **  Copyright (C) Sistina Software, Inc.  1997-2003  All rights reserved.
-**  Copyright (C) 2004 Red Hat, Inc.  All rights reserved.
+**  Copyright (C) 2004-2007 Red Hat, Inc.  All rights reserved.
 **
 **  This copyrighted material is made available to anyone wishing to use,
 **  modify, copy, or redistribute it subject to the terms and conditions
@@ -29,6 +29,7 @@
 #include "inode.h"
 #include "lm.h"
 #include "mount.h"
+#include "ops_dentry.h"
 #include "ops_export.h"
 #include "ops_fstype.h"
 #include "ops_super.h"
@@ -492,6 +493,7 @@ fill_super(struct super_block *sb, void *data, int silent)
 		error = -ENOMEM;
 		goto fail_li_free;
 	}
+	sb->s_root->d_op = &gfs_dops;
 
 	/*  Start up the logd thread  */
 
@@ -679,9 +681,8 @@ gfs_set_bdev_super(struct super_block *sb, void *data)
  * Returns: the new superblock
  */
 
-struct super_block *
-gfs_get_sb(struct file_system_type *fs_type, int flags,
-	   const char *dev_name, void *data)
+static int gfs_get_sb(struct file_system_type *fs_type, int flags,
+		      const char *dev_name, void *data, struct vfsmount *mnt)
 {
 	struct block_device *real, *diaper;
 	struct super_block *sb;
@@ -689,27 +690,29 @@ gfs_get_sb(struct file_system_type *fs_type, int flags,
 
 	real = open_bdev_excl(dev_name, flags, fs_type);
 	if (IS_ERR(real))
-		return (struct super_block *)real;
+		return PTR_ERR(real);
 
 	diaper = gfs_diaper_get(real, flags);
 	if (IS_ERR(diaper)) {
 		close_bdev_excl(real);
-		return (struct super_block *)diaper;
+		return PTR_ERR(diaper);
 	}
 
 	down(&diaper->bd_mount_sem);
 	sb = sget(fs_type, gfs_test_bdev_super, gfs_set_bdev_super, diaper);
 	up(&diaper->bd_mount_sem);
 	if (IS_ERR(sb))
-		goto out;
+		goto error_s;
 
 	if (sb->s_root) {
 		if ((flags ^ sb->s_flags) & MS_RDONLY) {
 			up_write(&sb->s_umount);
 			deactivate_super(sb);
-			sb = ERR_PTR(-EBUSY);
+			error = -EBUSY;
+			goto error_bdev;
 		}
-		goto out;
+
+		close_bdev_excl(real);
 	} else {
 		char buf[BDEVNAME_SIZE];
 
@@ -720,17 +723,27 @@ gfs_get_sb(struct file_system_type *fs_type, int flags,
 		if (error) {
 			up_write(&sb->s_umount);
 			deactivate_super(sb);
-			sb = ERR_PTR(error);
-		} else
-			sb->s_flags |= MS_ACTIVE;
+			goto error;
+		}
+		sb->s_flags |= MS_ACTIVE;
+		/* Equivilant of bdev_uevent(bdev, KOBJ_MOUNT): */
+		if (real->bd_disk) {
+			if (real->bd_part)
+				kobject_uevent(&real->bd_part->kobj, KOBJ_MOUNT);
+			else
+				kobject_uevent(&real->bd_disk->kobj, KOBJ_MOUNT);
+		}
 	}
 
-	return sb;
+	return simple_set_mnt(mnt, sb);
 
- out:
-	gfs_diaper_put(diaper);
+error_s:
+	error = PTR_ERR(sb);
+error_bdev:
 	close_bdev_excl(real);
-	return sb;
+error:
+	gfs_diaper_put(diaper);
+	return error;
 }
 
 /**
@@ -748,10 +761,18 @@ gfs_kill_sb(struct super_block *sb)
 	struct block_device *real = gfs_diaper_2real(diaper);
 	unsigned long bsize = block_size(real);
 
+	/* Equivalent of bdev_uevent(bdev, KOBJ_UMOUNT); */
+	if (real->bd_disk) {
+		if (real->bd_part)
+			kobject_uevent(&real->bd_part->kobj, KOBJ_UMOUNT);
+		else
+			kobject_uevent(&real->bd_disk->kobj, KOBJ_UMOUNT);
+	}
 	generic_shutdown_super(sb);
 	set_blocksize(diaper, bsize);
 	set_blocksize(real, bsize);
 	gfs_diaper_put(diaper);
+	sync_blockdev(real);
 	close_bdev_excl(real);
 }
 
