@@ -260,7 +260,10 @@ gfs_find_jhead(struct gfs_sbd *sdp, struct gfs_jindex *jdesc,
 
 		if (seg1 == seg_m) {
 			error = verify_jhead(sdp, jdesc, gl, &lh1);
-			memcpy(head, &lh1, sizeof(struct gfs_log_header));
+			if (unlikely(error)) 
+				printk("GFS: verify_jhead error=%d\n", error);
+			else
+				memcpy(head, &lh1, sizeof(struct gfs_log_header));
 			break;
 		}
 
@@ -545,7 +548,7 @@ clean_journal(struct gfs_sbd *sdp, struct gfs_jindex *jdesc,
 }
 
 /**
- * gfs_recover_journal - recovery a given journal
+ * gfs_recover_journal - recover a given journal
  * @sdp: the filesystem
  * @jid: the number of the journal to recover
  * @jdesc: the struct gfs_jindex describing the journal
@@ -570,7 +573,7 @@ gfs_recover_journal(struct gfs_sbd *sdp,
 	printk("GFS: fsid=%s: jid=%u: Trying to acquire journal lock...\n",
 	       sdp->sd_fsname, jid);
 
-	/*  Aquire the journal lock so we can do recovery  */
+	/*  Acquire the journal lock so we can do recovery  */
 
 	error = gfs_glock_nq_num(sdp,
 				 jdesc->ji_addr, &gfs_meta_glops,
