@@ -258,7 +258,7 @@ start_gulm_threads (char *csnm, char *hostdata)
 		if (hostdata != NULL && strlen (hostdata) > 0) {
 			strncpy (gulm_cm.myName, hostdata, 64);
 		} else {
-			strncpy (gulm_cm.myName, system_utsname.nodename, 64);
+			strncpy (gulm_cm.myName, init_utsname()->nodename, 64);
 		}
 		gulm_cm.myName[63] = '\0';
 
