@@ -879,11 +879,12 @@ int pass1(struct fsck_sb *sbp)
 		log_info("Checking metadata in Resource Group %"PRIu64"\n",
 			 rg_count);
 		rgd = osi_list_entry(tmp, struct fsck_rgrp, rd_list);
-		if(fs_rgrp_read(rgd)){
+		if(fs_rgrp_read(rgd, FALSE)){
 			stack;
 			return -1;
 		}
-		log_debug("RG at %"PRIu64" is %u long\n", rgd->rd_ri.ri_addr, rgd->rd_ri.ri_length);
+		log_debug("RG at %"PRIu64" is %u long\n", rgd->rd_ri.ri_addr,
+				  rgd->rd_ri.ri_length);
 		for (i = 0; i < rgd->rd_ri.ri_length; i++) {
 			if(block_set(sbp->bl, rgd->rd_ri.ri_addr + i,
 				     meta_other)){
@@ -903,7 +904,14 @@ int pass1(struct fsck_sb *sbp)
 			if(next_rg_meta_free(rgd, &block, first, &mfree))
 				break;
 
-
+			warm_fuzzy_stuff(block);
+			if (fsck_abort) /* if asked to abort */
+				return 0;
+			if (skip_this_pass) {
+				printf("Skipping pass 1 is not a good idea.\n");
+				skip_this_pass = FALSE;
+				fflush(stdout);
+			}
 			if(get_and_read_buf(sbp, block, &bh, 0)){
 				stack;
 				log_crit("Unable to retrieve block %"PRIu64
