@@ -738,6 +738,7 @@ static int master_probe_bottom(void)
 {
    Login_state.current = LLi_next(Login_state.current);
    Login_state.testing = FALSE;
+   int err;
 
    if( LLi_data(Login_state.current) == NULL &&
        I_am_the == gio_Mbr_ama_Pending ) {
@@ -786,7 +787,13 @@ static int master_probe_bottom(void)
          log_msg(lgm_Stomith, "LastMaster %s, is being marked Expired.\n",
                print_ipname(Login_state.LastMasterIN));
 
-         Mark_Expired(Login_state.LastMasterIN->name);
+         err = Mark_Expired(Login_state.LastMasterIN->name);
+         if (err)
+            err = Mark_Expired_IP(&Login_state.LastMasterIN->ip);
+         if (err)
+            log_msg(lgm_Always, "Mark_Expired failed on %s-%s",
+                  Login_state.LastMasterIN->name,
+                  ip6tostr(&Login_state.LastMasterIN->ip));
          send_mbrshp_to_children(Login_state.LastMasterIN->name,
                gio_Mbr_Expired);
          send_mbrshp_to_slaves(Login_state.LastMasterIN->name,
@@ -1982,11 +1989,7 @@ void work_loop(void)
 
    while( running ) {
 
-      /* the waitpid won't block, so depending on what adam sees with
-       * Attachment #158 on Bug #1288  I want to try removing the SIGCHLD
-       * completely.  And just whipover the waitpid everysingle time.
-       */
-      if( SIGCHLD_TRIPPED ) scan_for_dead_children();
+      scan_for_dead_children();
 
       if( I_am_the == gio_Mbr_ama_Slave) {
          if( send_heartbeat() != 0) {
