@@ -121,8 +121,8 @@ int svc_fail(char *svcName);
 int rt_enqueue_request(const char *resgroupname, int request, int response_fd,
        		       int max, uint64_t target, int arg0, int arg1);
 
-void send_response(int ret, request_t *req);
-void send_ret(int fd, char *name, int ret, int req);
+void send_response(int ret, uint64_t owner, request_t *req);
+void send_ret(int fd, char *name, int ret, int req, uint64_t newowner);
 
 /* do this op on all resource groups.  The handler for the request 
    will sort out whether or not it's a valid request given the state */
@@ -156,12 +156,14 @@ void member_list_update(cluster_member_list_t *new_ml);
 cluster_member_list_t *member_list(void);
 uint64_t my_id(void);
 
-#define RG_EAGAIN	-6
-#define RG_EDEADLCK	-5
-#define RG_ENOSERVICE	-4
-#define RG_EFORWARD	-3
-#define RG_EABORT	-2
-#define RG_EFAIL	-1
+#define RG_ENODEDEATH	-8 /* Processing node died */
+#define RG_ERUN		-7 /* Service is running already */
+#define RG_EAGAIN	-6 /* Try again */
+#define RG_EDEADLCK	-5 /* Operation would cause deadlock */
+#define RG_ENOSERVICE	-4 /* Service does not exist */
+#define RG_EFORWARD	-3 /* Ask current service owner to do this, please */
+#define RG_EABORT	-2 /* Request cancelled */
+#define RG_EFAIL	-1 /* Generic error */
 #define RG_ESUCCESS	0
 
 
