@@ -29,6 +29,9 @@
 #include <list.h>
 #include <reslist.h>
 #include <pthread.h>
+#ifndef NO_CCS
+#include <clulog.h>
+#endif
 
 
 char *attr_value(resource_node_t *node, char *attrname);
@@ -77,6 +80,8 @@ attr_value(resource_node_t *node, char *attrname)
 {
 	resource_t *res = node->rn_resource;
 	resource_attr_t *ra;
+	char *c, *p_type;
+	ssize_t len;
 	int x;
 
 	for (x = 0; res->r_attrs && res->r_attrs[x].ra_name; x++) {
@@ -85,10 +90,34 @@ attr_value(resource_node_t *node, char *attrname)
 
 		ra = &res->r_attrs[x];
 
-		if (ra->ra_flags & RA_INHERIT)
+		if (!(ra->ra_flags & RA_INHERIT))
+			return ra->ra_value;
+		/* 
+		   Handle resource_type%field to be more precise, so we
+		   don't have to worry about this being a child
+		   of an unexpected type.  E.g. lots of things have the
+		   "name" attribute.
+		 */
+		c = strchr(ra->ra_value, '%');
+		if (!c) {
+			/* Someone doesn't care or uses older
+			   semantics on inheritance */
 			return attr_value(node->rn_parent, ra->ra_value);
+		}
 
-		return ra->ra_value;
+		p_type = node->rn_parent->rn_resource->r_rule->rr_type;
+		len = (c - ra->ra_value);
+
+		/* Different sizes? */
+		if (strlen(p_type) != len)
+			return NULL;
+
+		/* Check for parent type match */
+		if (strncmp(p_type, ra->ra_value, len) != 0)
+			return NULL;
+
+		/* Skip the "%" and recurse */
+		return attr_value(node->rn_parent, ++c);
 	}
 
 	return NULL;
@@ -313,12 +342,29 @@ store_resource(resource_t **reslist, resource_t *newres)
 					/*
 					   Unique/primary is not unique
 					 */
-					printf("Unique/primary not unique "
-					       "type %s, %s=%s\n",
+#ifdef NO_CCS
+					printf("Error: "
+                                               "%s attribute collision. "
+                                               "type=%s attr=%s value=%s\n",
+					       (newres->r_attrs[x].ra_flags&
+                                                RA_PRIMARY)?"Primary":
+                                               "Unique",
 					       newres->r_rule->rr_type,
 					       newres->r_attrs[x].ra_name,
 					       newres->r_attrs[x].ra_value
 					       );
+#else 
+					clulog(LOG_ERR,
+                                               "%s attribute collision. "
+                                               "type=%s attr=%s value=%s\n",
+					       (newres->r_attrs[x].ra_flags&
+                                                RA_PRIMARY)?"Primary":
+                                               "Unique",
+					       newres->r_rule->rr_type,
+					       newres->r_attrs[x].ra_name,
+					       newres->r_attrs[x].ra_value
+					       );
+#endif
 					return -1;
 				}
 				break;
@@ -497,6 +543,83 @@ act_dup(resource_act_t *acts)
 }
 
 
+/* Copied from resrules.c -- _get_actions */
+void
+_get_actions_ccs(int ccsfd, char *base, resource_t *res)
+{
+	char xpath[256];
+	int idx = 0;
+	char *act, *ret;
+	int interval, timeout, depth;
+
+	do {
+		/* setting these to -1 prevents overwriting with 0 */
+		interval = -1;
+		depth = -1;
+		act = NULL;
+		timeout = -1;
+
+		snprintf(xpath, sizeof(xpath),
+			 "%s/action[%d]/@name", base, ++idx);
+
+#ifndef NO_CCS
+		if (ccs_get(ccsfd, xpath, &act) != 0)
+#else
+		if (conf_get(xpath, &act) != 0)
+#endif
+			break;
+
+		snprintf(xpath, sizeof(xpath),
+			 "%s/action[%d]/@timeout", base, idx);
+#ifndef NO_CCS
+		if (ccs_get(ccsfd, xpath, &ret) == 0 && ret) {
+#else
+		if (conf_get(xpath, &ret) == 0 && ret) {
+#endif
+			timeout = expand_time(ret);
+			if (timeout < 0)
+				timeout = 0;
+			free(ret);
+		}
+
+		snprintf(xpath, sizeof(xpath),
+			 "%s/action[%d]/@interval", base, idx);
+#ifndef NO_CCS
+		if (ccs_get(ccsfd, xpath, &ret) == 0 && ret) {
+#else
+		if (conf_get(xpath, &ret) == 0 && ret) {
+#endif
+			interval = expand_time(ret);
+			if (interval < 0)
+				interval = 0;
+			free(ret);
+		}
+
+		if (!strcmp(act, "status") || !strcmp(act, "monitor")) {
+			snprintf(xpath, sizeof(xpath),
+				 "%s/action[%d]/@depth", base, idx);
+#ifndef NO_CCS
+			if (ccs_get(ccsfd, xpath, &ret) == 0 && ret) {
+#else
+			if (conf_get(xpath, &ret) == 0 && ret) {
+#endif
+				depth = atoi(ret);
+				if (depth < 0)
+					depth = 0;
+				
+				/* */
+				if (ret[0] == '*')
+					depth = -1;
+				free(ret);
+			}
+		}
+
+		if (store_action(&res->r_actions, act, depth, timeout,
+				 interval) != 0)
+			free(act);
+	} while (1);
+}
+
 
 /**
    Try to load all the attributes in our rule set.  If none are found,
@@ -602,12 +725,12 @@ load_resource(int ccsfd, resource_rule_t *rule, char *base)
 	}
 
 	if (!found) {
-		//printf("No attributes found for %s\n", base);
 		destroy_resource(res);
 		return NULL;
 	}
 
 	res->r_actions = act_dup(rule->rr_actions);
+	_get_actions_ccs(ccsfd, base, res);
 
 	return res;
 }
@@ -634,14 +757,20 @@ load_resources(int ccsfd, resource_t **reslist, resource_rule_t **rulelist)
 		for (resID = 1; ; resID++) {
 			snprintf(tok, sizeof(tok), RESOURCE_BASE "/%s[%d]",
 				 currule->rr_type, resID);
-
+			
 			newres = load_resource(ccsfd, currule, tok);
 			if (!newres)
 				break;
 
 		       if (store_resource(reslist, newres) != 0) {
+#ifdef NO_CCS
 	       		       printf("Error storing %s resource\n",
 				      newres->r_rule->rr_type);
+#else
+	       		       clulog(LOG_ERR,
+				      "Error storing %s resource\n",
+				      newres->r_rule->rr_type);
+#endif
 			       destroy_resource(newres);
 		       }
 
