@@ -48,8 +48,9 @@ forwarding_thread(void *arg)
 	rg_state_t rgs;
 	request_t *req = (request_t *)arg;
 	void *lockp;
-	int fd;
+	int fd, ret;
 	SmMessageSt msg;
+	cluster_member_list_t *m = NULL;
 
 	if (rg_lock(req->rr_group, &lockp) != 0) {
 		msg_close(req->rr_resp_fd);
@@ -88,19 +89,43 @@ forwarding_thread(void *arg)
 		pthread_exit(NULL);
 	}
 
-	if (msg_receive(fd, &msg, sizeof(msg)) != sizeof(msg)) {
-		msg_close(fd);
-		msg_close(req->rr_resp_fd);
-		rq_free(req);
-		pthread_exit(NULL);
-	}
+	/*
+	 * Ok, we're forwarding a message to another node.  Keep tabs on 
+	 * the node to make sure it doesn't die.  Basically, wake up every
+	 * now and again to make sure it's still online.  If it isn't, send
+	 * a response back to the caller.
+	 */
+	do {
+		ret = msg_receive_timeout(fd, &msg, sizeof(msg), 10);
+		if (ret < (int)sizeof(msg)) {
+			if (ret < 0 && errno == ETIMEDOUT) {
+				m = member_list();
+				if (!memb_online(m, rgs.rs_owner)) {
+					msg.sm_data.d_ret = RG_ENODEDEATH;
+					/* we decode down below,
+					 * so encode here */
+					swab_SmMessageSt(&msg);
+					break;
+				}
+				cml_free(m);
+				m = NULL;
+				continue;
+			}
+			msg_close(fd);
+			msg_close(req->rr_resp_fd);
+			goto out;
+		}
+		break;
+	} while(1);
+
+	if (m)
+		cml_free(m);
 	msg_close(fd);
 
 	swab_SmMessageSt(&msg);
-	send_response(msg.sm_data.d_ret, req);
-
+	send_response(msg.sm_data.d_ret, req->rr_target, req);
+out:
 	rq_free(req);
-
 	pthread_exit(NULL);
 }
 
