@@ -110,75 +110,6 @@ svc_report_failure(char *svcName)
 }
 
 
-int
-clu_lock_verbose(char *resource, int dflt_flags, void **lockpp)
-{
-	int ret, timed_out = 0;
-	struct timeval start, now;
-	uint64_t nodeid, *p;
-	int flags;
-	int block = !(dflt_flags & CLK_NOWAIT);
-
-	/* Holder not supported for this call */
-	dflt_flags &= ~CLK_HOLDER;
-
-	flags = dflt_flags;
-
-	if (block) {
-		gettimeofday(&start, NULL);
-		start.tv_sec += 30;
-	}
-	while (1) {
-		if (block) {
-			gettimeofday(&now, NULL);
-
-			if ((now.tv_sec > start.tv_sec) ||
-			    ((now.tv_sec == start.tv_sec) &&
-	 		     (now.tv_usec >= start.tv_usec))) {
-
-				gettimeofday(&start, NULL);
-				start.tv_sec += 30;
-
-				timed_out = 1;
-				flags |= CLK_HOLDER;
-			}
-		}
-
-		ret = clu_lock(resource, flags | CLK_NOWAIT, lockpp);
-
-		if ((ret != 0) && (errno == EAGAIN) && block) {
-			if (timed_out) {
-				p = (uint64_t *)*lockpp;
-				if (p) {
-					nodeid = *p;
-					clulog(LOG_WARNING, "Node ID:%08x%08x"
-					       " stuck with lock %s\n",
-					       (uint32_t)(nodeid>>32&0xffffffff),
-					       (uint32_t)nodeid&0xffffffff,
-					       resource);
-					free(p);
-				} else {
-					clulog(LOG_WARNING, "Starving for lock"
-					       " %s\n", resource);
-				}
-				flags = dflt_flags;
-				timed_out = 0;
-			}
-			usleep(random()&32767<<1);
-			continue;
-
-		} else if (ret == 0) {
-			/* Success */
-			return 0;
-		}
-
-		break;
-	}
-
-	return ret;
-}
-
-
 int
 #ifdef DEBUG
 _rg_lock(char *name, void **p)
@@ -189,7 +120,7 @@ rg_lock(char *name, void **p)
 	char res[256];
 
 	snprintf(res, sizeof(res), "usrm::rg=\"%s\"", name);
-	return clu_lock_verbose(res, CLK_EX, p);
+	return clu_lock(res, CLK_EX, p);
 }
 
 
@@ -229,7 +160,7 @@ _rg_unlock_dbg(char *name, void *p, char *file, int line)
 
 
 void
-send_ret(int fd, char *name, int ret, int orig_request)
+send_ret(int fd, char *name, int ret, int orig_request, uint64_t newowner)
 {
 	SmMessageSt msg, *msgp = &msg;
 	if (fd < 0)
@@ -241,7 +172,11 @@ send_ret(int fd, char *name, int ret, int orig_request)
 	msgp->sm_data.d_action = orig_request;
 	strncpy(msgp->sm_data.d_svcName, name,
 		sizeof(msgp->sm_data.d_svcName));
-	msgp->sm_data.d_svcOwner = my_id(); /* XXX Broken */
+	if (newowner == NODE_ID_NONE) {
+		msgp->sm_data.d_svcOwner = my_id(); /* XXX Broken */
+	} else { 
+		msgp->sm_data.d_svcOwner = newowner;
+	}
 	msgp->sm_data.d_ret = ret;
 
 	swab_SmMessageSt(msgp);
@@ -253,7 +188,7 @@ send_ret(int fd, char *name, int ret, int orig_request)
 
 	
 void
-send_response(int ret, request_t *req)
+send_response(int ret, uint64_t newowner, request_t *req)
 {
 	SmMessageSt msg, *msgp = &msg;
 
@@ -266,7 +201,11 @@ send_response(int ret, request_t *req)
 	msgp->sm_data.d_action = req->rr_orig_request;
 	strncpy(msgp->sm_data.d_svcName, req->rr_group,
 		sizeof(msgp->sm_data.d_svcName));
-	msgp->sm_data.d_svcOwner = my_id(); /* XXX Broken */
+	if (newowner == NODE_ID_NONE) {
+		msgp->sm_data.d_svcOwner = my_id(); /* XXX Broken */
+	} else { 
+		msgp->sm_data.d_svcOwner = newowner;
+	}
 	msgp->sm_data.d_ret = ret;
 
 	swab_SmMessageSt(msgp);
@@ -369,6 +308,46 @@ get_rg_state(char *name, rg_state_t *svcblk)
 }
 
 
+int vf_read_local(char *, uint64_t *, void *, uint32_t *);
+int
+get_rg_state_local(char *name, rg_state_t *svcblk)
+{
+	char res[256];
+	int ret;
+	void *data = NULL;
+	uint32_t datalen = 0;
+	uint64_t viewno;
+
+	/* ... */
+	if (name)
+		strncpy(svcblk->rs_name, name, sizeof(svcblk->rs_name));
+
+	snprintf(res, sizeof(res),"usrm::rg=\"%s\"", svcblk->rs_name);
+	ret = vf_read_local(res, &viewno, &data, &datalen);
+
+	if (ret != VFR_OK || datalen == 0 ||
+	    datalen != sizeof(*svcblk)) {
+		if (data)
+			free(data);
+
+		svcblk->rs_owner = NODE_ID_NONE;
+		svcblk->rs_last_owner = NODE_ID_NONE;
+		svcblk->rs_state = RG_STATE_UNINITIALIZED;
+       		svcblk->rs_restarts = 0;
+		svcblk->rs_transition = 0;	
+		strncpy(svcblk->rs_name, name, sizeof(svcblk->rs_name));
+
+		return FAIL;
+	}
+
+	/* Copy out the data. */
+	memcpy(svcblk, data, sizeof(*svcblk));
+	free(data);
+
+	return 0;
+}
+
+
 /**
  * Advise service manager as to whether or not to stop a service, given
  * that we already know it's legal to run the service.
@@ -380,6 +359,7 @@ get_rg_state(char *name, rg_state_t *svcblk)
  *			1 = STOP service - return whatever it returns.
  *			2 = DO NOT stop service, return 0 (success)
  *                      3 = DO NOT stop service, return RG_EFORWARD
+ *			4 = DO NOT stop service, return RG_EAGAIN
  */
 int
 svc_advise_stop(rg_state_t *svcStatus, char *svcName, int req)
@@ -396,10 +376,6 @@ svc_advise_stop(rg_state_t *svcStatus, char *svcName, int req)
 		break;
 		
 	case RG_STATE_STOPPING:
-		printf("Service %s is stopping right now\n", svcName);
-		ret = 2;
-		break;
-
 	case RG_STATE_STARTED:
 	case RG_STATE_CHECK:
 	case RG_STATE_STARTING:
@@ -504,6 +480,8 @@ svc_advise_stop(rg_state_t *svcStatus, char *svcName, int req)
  * @return		0 = DO NOT start service, return FAIL
  *			1 = START service - return whatever it returns.
  *			2 = DO NOT start service, return 0
+ *			3 = DO NOT start service, return RG_EAGAIN
+ *                      4 = DO NOT start servuce, return RG_ERUN
  */
 int
 svc_advise_start(rg_state_t *svcStatus, char *svcName, int req)
@@ -519,10 +497,6 @@ svc_advise_start(rg_state_t *svcStatus, char *svcName, int req)
 		break;
 		
 	case RG_STATE_STOPPING:
-		clulog(LOG_DEBUG, "RG %s is stopping\n", svcName);
-		ret = 2;
-		break;
-
 	case RG_STATE_STARTED:
 	case RG_STATE_CHECK:
 	case RG_STATE_STARTING:
@@ -532,7 +506,7 @@ svc_advise_start(rg_state_t *svcStatus, char *svcName, int req)
 			clulog(LOG_DEBUG,
 			       "RG %s is already running locally\n", svcName);
 			 */
-			ret = 2;
+			ret = 4;
 			break;
 		}
 
@@ -544,11 +518,18 @@ svc_advise_start(rg_state_t *svcStatus, char *svcName, int req)
 			       svcName,
 			       memb_id_to_name(membership,svcStatus->rs_owner));
 			 */
-			ret = 2;
+			ret = 4;
 			break;
 		}
 
+		/* We are allowed to do something with the service.  Make
+		   sure we're not locked */
 		if (svcStatus->rs_owner == NODE_ID_NONE) {
+			if (rg_locked()) {
+				ret = 3;
+				break;
+			}
+
 			clulog(LOG_NOTICE,
 			       "Starting stopped service%s\n",
 			       svcName);
@@ -556,6 +537,13 @@ svc_advise_start(rg_state_t *svcStatus, char *svcName, int req)
 			break;
 		}
 
+		if (rg_locked()) {
+			clulog(LOG_WARNING, "Not initiating failover of %s: "
+			       "Resource groups locked!\n", svcName);
+			ret = 3;
+			break;
+		}
+
 		/*
 		 * Service is running but owner is down -> FAILOVER
 		 */
@@ -588,6 +576,13 @@ svc_advise_start(rg_state_t *svcStatus, char *svcName, int req)
 		break;
 
 	case RG_STATE_STOPPED:
+	case RG_STATE_ERROR:
+		/* Don't actually enable if the RG is locked! */
+		if (rg_locked()) {
+			ret = 3;
+			break;
+		}
+
 		clulog(LOG_NOTICE, "Starting stopped service %s\n",
 		       svcName);
 		ret = 1;
@@ -596,6 +591,12 @@ svc_advise_start(rg_state_t *svcStatus, char *svcName, int req)
 	case RG_STATE_DISABLED:
 	case RG_STATE_UNINITIALIZED:
 		if (req == RG_ENABLE) {
+			/* Don't actually enable if the RG is locked! */
+			if (rg_locked()) {
+				ret = 3;
+				break;
+			}
+
 			clulog(LOG_NOTICE,
 			       "Starting disabled service %s\n",
 			       svcName);
@@ -607,7 +608,6 @@ svc_advise_start(rg_state_t *svcStatus, char *svcName, int req)
 		       svcName);
 		break;
 
-	case RG_STATE_ERROR:
 	default:
 		clulog(LOG_ERR,
 		       "#44: Cannot start RG %s: Invalid State %d\n",
@@ -656,6 +656,12 @@ svc_start(char *svcName, int req)
 	case 2: /* Don't start service, return 0 */
 		rg_unlock(svcName, lockp);
 		return 0;
+	case 3:
+		rg_unlock(svcName, lockp);
+		return RG_EAGAIN;
+	case 4:
+		rg_unlock(svcName, lockp);
+		return RG_ERUN;
 	default:
 		break;
 	}
@@ -738,14 +744,12 @@ svc_status(char *svcName)
 	}
 	rg_unlock(svcName, lockp);
 
-	if (svcStatus.rs_state == RG_STATE_STARTED &&
-	    svcStatus.rs_owner != my_id())
-		/* Don't check status for other resource groups */
+	if (svcStatus.rs_owner != my_id())
+		/* Don't check status for anything not owned */
 		return SUCCESS;
 
-	if (svcStatus.rs_state != RG_STATE_STARTED &&
-	    svcStatus.rs_owner == my_id())
-		/* Not-running RGs should not be checked yet. */
+	if (svcStatus.rs_state != RG_STATE_STARTED)
+		/* Not-running RGs should not be checked either. */
 		return SUCCESS;
 
 	return group_op(svcName, RG_STATUS);
@@ -766,6 +770,7 @@ _svc_stop(char *svcName, int req, int recover, uint32_t newstate)
 	void *lockp = NULL;
 	rg_state_t svcStatus;
 	int ret;
+	int old_state;
 
 	if (!rg_quorate()) {
 		clulog(LOG_WARNING, "#69: Unclean %s of %s\n", 
@@ -798,10 +803,15 @@ _svc_stop(char *svcName, int req, int recover, uint32_t newstate)
 	case 3:
 		rg_unlock(svcName, lockp);
 		return RG_EFORWARD;
+	case 4:
+		rg_unlock(svcName, lockp);
+		return RG_EAGAIN;
 	default:
 		break;
 	}
 
+	old_state = svcStatus.rs_state;
+
 	clulog(LOG_NOTICE, "Stopping service %s\n", svcName);
 
 	if (recover)
@@ -821,7 +831,15 @@ _svc_stop(char *svcName, int req, int recover, uint32_t newstate)
 
 	ret = group_op(svcName, RG_STOP);
 
-	_svc_stop_finish(svcName, ret, newstate);
+	if (old_state == RG_STATE_FAILED && newstate == RG_STATE_DISABLED) {
+		if (ret)
+			clulog(LOG_ALERT, "Marking %s as 'disabled', "
+			       "but some resources may still be allocated!\n",
+			       svcName);
+		_svc_stop_finish(svcName, 0, newstate);
+	} else {
+		_svc_stop_finish(svcName, ret, newstate);
+	}
 
 	return ret;
 }
@@ -1077,7 +1095,7 @@ int
 handle_relocate_req(char *svcName, int request, uint64_t preferred_target,
 		    uint64_t *new_owner)
 {
-	cluster_member_list_t *allowed_nodes;
+	cluster_member_list_t *allowed_nodes, *backup = NULL;
 	uint64_t target = preferred_target, me = my_id();
 	int ret, x;
 	
@@ -1102,19 +1120,23 @@ handle_relocate_req(char *svcName, int request, uint64_t preferred_target,
 		   If we can't start it on the preferred target, then we'll try
 	 	   other nodes.
 		 */
+		//count_resource_groups(allowed_nodes);
+		backup = cml_dup(allowed_nodes);
+
 		for (x = 0; x < allowed_nodes->cml_count; x++) {
 			if (allowed_nodes->cml_members[x].cm_id == me ||
-		    	    allowed_nodes->cml_members[x].cm_id == preferred_target)
+		    	    allowed_nodes->cml_members[x].cm_id ==
+			    		preferred_target)
 				continue;
 			allowed_nodes->cml_members[x].cm_state = STATE_DOWN;
 		}
 
 		/*
-		 * First, see if it's legal to relocate to the target node.  Legal
-		 * means: the node is online and is in the [restricted] failover
-		 * domain of the service, or the service has no failover domain.
+		 * First, see if it's legal to relocate to the target node.
+		 * Legal means: the node is online and is in the
+		 * [restricted] failover domain of the service, or the
+		 * service has no failover domain.
 		 */
-
 		target = best_target_node(allowed_nodes, me, svcName, 1);
 
 		cml_free(allowed_nodes);
@@ -1155,7 +1177,12 @@ handle_relocate_req(char *svcName, int request, uint64_t preferred_target,
 	 * Ok, so, we failed to send it to the preferred target node.
 	 * Try to start it on all other nodes.
 	 */
-	allowed_nodes = member_list();
+	if (backup) {
+		allowed_nodes = backup;
+	} else {
+		allowed_nodes = member_list();
+		//count_resource_groups(allowed_nodes);
+	}
 
 	if (preferred_target != NODE_ID_NONE)
 		memb_mark_down(allowed_nodes, preferred_target);
@@ -1208,12 +1235,14 @@ handle_relocate_req(char *svcName, int request, uint64_t preferred_target,
 	 * We're done.
 	 */
 exhausted:
-	clulog(LOG_WARNING,
-	       "#70: Attempting to restart service %s locally.\n",
-	       svcName);
-	if (svc_start(svcName, RG_START_RECOVER) == 0) {
-		*new_owner = me;
-		return FAIL;
+	if (!rg_locked()) {
+		clulog(LOG_WARNING,
+		       "#70: Attempting to restart service %s locally.\n",
+		       svcName);
+		if (svc_start(svcName, RG_START_RECOVER) == 0) {
+			*new_owner = me;
+			return FAIL;
+		}
 	}
 
 	if (svc_stop(svcName, RG_STOP) != 0) {
@@ -1225,6 +1254,60 @@ exhausted:
 }
 
 
+int
+handle_fd_start_req(char *svcName, int request, uint64_t *new_owner)
+{
+	cluster_member_list_t *allowed_nodes, *backup = NULL;
+	uint64_t target, me = my_id();
+	int ret, x;
+	
+	allowed_nodes = member_list();
+
+	while (memb_count(allowed_nodes)) {
+		target = best_target_node(allowed_nodes, NODE_ID_NONE,
+					  svcName, 1);
+		if (target == me) {
+			ret = handle_start_remote_req(svcName, request);
+		} else {
+			ret = relocate_service(svcName, request, target);
+		}
+
+		switch (ret) {
+		case RG_ERUN:
+			return RG_ERUN;
+		case RG_EFAIL:
+			memb_mark_down(allowed_nodes, target);
+			continue;
+		case RG_EABORT:
+			svc_report_failure(svcName);
+			cml_free(allowed_nodes);
+			return FAIL;
+		case NO:
+			/* state uncertain */
+			cml_free(allowed_nodes);
+			clulog(LOG_DEBUG, "State Uncertain: svc:%s "
+			       "nid:%08x%08x req:%d\n", svcName,
+			       (uint32_t)(target>>32)&0xffffffff,
+			       (uint32_t)(target&0xffffffff), request);
+			return 0;
+		case 0:
+			*new_owner = target;
+			clulog(LOG_NOTICE, "Service %s is now running "
+			       "on member %d\n", svcName, (int)target);
+			cml_free(allowed_nodes);
+			return 0;
+		default:
+			clulog(LOG_ERR,
+			       "#61: Invalid reply from member %d during"
+			       " relocate operation!\n", target);
+		}
+	}
+	
+	cml_free(allowed_nodes);
+	return RG_EFAIL;
+}
+
+
 /**
  * handle_start_req - Handle a generic start request from a user or during
  * service manager boot.
@@ -1263,6 +1346,12 @@ handle_start_req(char *svcName, int req, uint64_t *new_owner)
 	 */
 	ret = svc_start(svcName, req);
 
+	/* 
+	   If services are locked, return the error 
+	  */
+	if (ret == RG_EAGAIN || ret == RG_ERUN)
+		return ret;
+
 	/*
 	 * If we succeeded, then we're done.
 	 */
@@ -1354,11 +1443,14 @@ handle_start_remote_req(char *svcName, int req)
 	}
 	cml_free(membership);
 
-	if (svc_start(svcName, req) == 0)
+	x = svc_start(svcName, req);
+	if (x == 0)
 		return 0;
+	if (x == RG_ERUN)
+		return RG_ERUN;
 
 	if (svc_stop(svcName, RG_STOP_RECOVER) == 0)
-		return FAIL;
+		return RG_EFAIL;
 
 	svc_fail(svcName);
 	return RG_EABORT;
