@@ -110,18 +110,22 @@ static void
 purge_status_checks(request_t **list)
 {
 	request_t *curr;
+	int found;
 	
 	if (!list)
 		return;
 
-	list_do(list, curr) {
-		if (curr->rr_request != RG_STATUS)
-			continue;
-
-		list_remove(list, curr);
-		rq_free(curr);
-		curr = *list;
-	} while (!list_done(list, curr));
+	do {
+		found = 0;
+		list_do(list, curr) {
+			if (curr->rr_request == RG_STATUS) {
+				list_remove(list, curr);
+				rq_free(curr);
+				found = 1;
+				break;
+			}
+		} while (!list_done(list, curr));
+	} while (found);
 }
 
 
@@ -141,7 +145,7 @@ purge_all(request_t **list)
 		list_remove(list, curr);
 		dprintf("Removed request %d\n", curr->rr_request);
 		if (curr->rr_resp_fd != -1) {
-			send_response(RG_EABORT, curr);
+			send_response(RG_EABORT, NODE_ID_NONE, curr);
 		}
 		rq_free(curr);
 	}
@@ -233,8 +237,15 @@ resgroup_thread_main(void *arg)
 
 		case RG_START:
 		case RG_ENABLE:
-			error = handle_start_req(myname, req->rr_request,
-						 &newowner);
+			if (req->rr_arg0) {
+				error = handle_fd_start_req(myname,
+						            req->rr_request,
+						            &newowner);
+			} else {
+				error = handle_start_req(myname,
+							 req->rr_request,
+							 &newowner);
+			}
 			break;
 
 		case RG_RELOCATE:
@@ -408,9 +419,8 @@ resgroup_thread_main(void *arg)
 
 		if (ret != RG_NONE && rg_initialized() &&
 		    (req->rr_resp_fd >= 0)) {
-			send_response(error, req);
+			send_response(error, newowner, req);
 		}
-
 		
 		rq_free(req);
 	}
@@ -607,7 +617,7 @@ rt_enqueue_request(const char *resgroupname, int request, int response_fd,
 		case RG_START:
 		case RG_ENABLE:
 			send_ret(response_fd, resgroup->rt_name, RG_EDEADLCK,
-				 request);
+				 request, NODE_ID_NONE);
 			break;
 		}
 		fprintf(stderr, "Failed to queue request: Would block\n");
