@@ -52,6 +52,7 @@ rg_state_list(uint64_t local_node_id, int fast)
 
 	fd = msg_open(local_node_id, RG_PORT, RG_PURPOSE, 10);
 	if (fd == -1) {
+		perror("msg_open");
 		return NULL;
 	}
 
@@ -234,9 +235,6 @@ add_missing(cluster_member_list_t *all, cluster_member_list_t *these)
 		}
 
 		if (!m) {
-			printf("%s not found\n", these->cml_members[x].cm_name);
-			/* WTF? It's not in our config */
-			printf("realloc %d\n", (int)cml_size((all->cml_count+1)));
 			all = realloc(all, cml_size((all->cml_count+1)));
 			if (!all) {
 				perror("realloc");
@@ -358,14 +356,14 @@ xml_rg_state(rg_state_t *rs, cluster_member_list_t *members, int flags)
 }
 
 
-void
+int
 txt_rg_states(rg_state_list_t *rgl, cluster_member_list_t *members, 
 	      char *svcname, int flags)
 {
-	int x;
+	int x, ret = 0;
 
 	if (!rgl || !members)
-		return;
+		return -1;
 
 	if (!(flags & RG_VERBOSE)) {
 		printf("  %-20.20s %-30.30s %-14.14s\n",
@@ -382,18 +380,31 @@ txt_rg_states(rg_state_list_t *rgl, cluster_member_list_t *members,
 		    strcmp(rgl->rgl_states[x].rs_name, svcname))
 			continue;
 		txt_rg_state(&rgl->rgl_states[x], members, flags);
+		if (svcname) {
+			switch (rgl->rgl_states[x].rs_state) {
+			case RG_STATE_STARTING:
+			case RG_STATE_STARTED:
+			case RG_STATE_STOPPING:
+				break;
+			default:
+				ret = rgl->rgl_states[x].rs_state;
+			}
+		}
 	}
+	
+	return ret;
 }
 
 
-void
+int
 xml_rg_states(rg_state_list_t *rgl, cluster_member_list_t *members,
-	      char *svcname)
+	      char *svcname, int flags)
 {
 	int x;
+	int ret = 0;
 
 	if (!rgl || !members)
-		return;
+		return -1;
 
 	printf("  <groups>\n");
 
@@ -401,14 +412,24 @@ xml_rg_states(rg_state_list_t *rgl, cluster_member_list_t *members,
 		if (svcname &&
 		    strcmp(rgl->rgl_states[x].rs_name, svcname))
 			continue;
-		xml_rg_state(&rgl->rgl_states[x], members, 0);
+		xml_rg_state(&rgl->rgl_states[x], members, flags);
+		if (svcname) {
+			switch (rgl->rgl_states[x].rs_state) {
+			case RG_STATE_STARTING:
+			case RG_STATE_STARTED:
+			case RG_STATE_STOPPING:
+				break;
+			default:
+				ret = rgl->rgl_states[x].rs_state;
+			}
+		}
 	}
 
 	printf("  </groups>\n");
+	return ret;
 }
 
 
-
 void
 txt_quorum_state(int qs)
 {
@@ -481,10 +502,15 @@ xml_member_state(cluster_member_t *node)
 }
 
 
-void
+int 
 txt_member_states(cluster_member_list_t *membership, char *name)
 {
-	int x;
+	int x, ret = 0;
+
+	if (!membership) {
+		printf("Membership information not available\n");
+		return -1;
+	}
 
 	printf("  %-40.40s %s\n", "Member Name", "Status");
 	printf("  %-40.40s %s\n", "------ ----", "------");
@@ -493,35 +519,45 @@ txt_member_states(cluster_member_list_t *membership, char *name)
 		if (name && strcmp(membership->cml_members[x].cm_name, name))
 			continue;
 		txt_member_state(&membership->cml_members[x]);
+		ret = !(membership->cml_members[x].cm_state & FLAG_UP);
 	}
 
 	printf("\n");
+	return ret;
 }
 
 
-void
+int 
 xml_member_states(cluster_member_list_t *membership, char *name)
 {
-	int x;
+	int x, ret = 0;
 
-	if (!membership)
-		return;
+	if (!membership) {
+		printf("  <nodes/>\n");
+		return -1;
+	}
 
 	printf("  <nodes>\n");
 	for (x = 0; x < membership->cml_count; x++) {
 		if (name && strcmp(membership->cml_members[x].cm_name, name))
 			continue;
 		xml_member_state(&membership->cml_members[x]);
+		if (name)
+			ret = !(membership->cml_members[x].cm_state & FLAG_UP);
 	}
 	printf("  </nodes>\n");
+	
+	return ret;
 }
 
 
-void
+int 
 txt_cluster_status(int qs, cluster_member_list_t *membership,
 		   rg_state_list_t *rgs, char *name, char *svcname, 
 		   int flags)
 {
+	int ret;
+	
 	if (!svcname && !name) {
 		txt_quorum_state(qs);
 		if (!membership || !(qs & QF_GROUPMEMBER)) {
@@ -531,50 +567,43 @@ txt_cluster_status(int qs, cluster_member_list_t *membership,
 	}
 
 	if (!svcname || (name && svcname))
-		txt_member_states(membership, name);
+		ret = txt_member_states(membership, name);
+	if (name && !svcname)
+		return ret;
 	if (!name || (name && svcname))
-		txt_rg_states(rgs, membership, svcname, flags);
+		ret = txt_rg_states(rgs, membership, svcname, flags);
+	return ret;
 }
 
 
-void
+int
 xml_cluster_status(int qs, cluster_member_list_t *membership,
 		   rg_state_list_t *rgs, char *name, char *svcname,
 		   int flags)
 {
+	int ret1 = 0, ret2 = -1;
+	
 	printf("<?xml version=\"1.0\"?>\n");
 	printf("<clustat version=\"4.1.1\">\n");
 
 	if (!svcname && !name)
 		xml_quorum_state(qs);
 	if (!svcname || (name && svcname)) 
-		xml_member_states(membership, name);
+		ret1 = xml_member_states(membership, name);
+	
 	if (rgs &&
 	    (!name || (name && svcname)))
-		xml_rg_states(rgs, membership, svcname);
+		ret2 = xml_rg_states(rgs, membership, svcname, flags);
 	printf("</clustat>\n");
+	
+	if (name && ret1)
+		return ret1;
+	if (svcname && ret2)
+		return ret2;
+	return 0;
 }
 
 
-void
-dump_node(cluster_member_t *node)
-{
-	printf("Node %s state %02x\n", node->cm_name, node->cm_state);
-}
-
-
-void 
-dump_nodes(cluster_member_list_t *nodes)
-{
-	int x;
-
-	for (x=0; x<nodes->cml_count; x++) {
-		dump_node(&nodes->cml_members[x]);
-	}
-}
-
-
-
 cluster_member_list_t *
 build_member_list(uint64_t *lid)
 {
@@ -608,6 +637,10 @@ build_member_list(uint64_t *lid)
 	/* Grab the local node ID and flag it from the list of reported
 	   online nodes */
 	clu_local_nodeid(NULL, lid);
+
+	if (!all)
+		return NULL;
+
 	for (x=0; x<all->cml_count; x++) {
 		if (all->cml_members[x].cm_id == *lid) {
 			m = &all->cml_members[x];
@@ -759,7 +792,8 @@ main(int argc, char **argv)
 		qs = clu_quorum_status(RG_SERVICE_GROUP);
 		membership = build_member_list(&local_node_id);
 		
-		rgs = rg_state_list(local_node_id, fast);
+		if (!member_name)
+			rgs = rg_state_list(local_node_id, fast);
 
 		if (refresh_sec) {
 			setupterm((char *) 0, STDOUT_FILENO, (int *) 0);
@@ -767,11 +801,13 @@ main(int argc, char **argv)
 		}
 
 		if (xml)
-			xml_cluster_status(qs, membership, rgs, member_name,
-					   rg_name,flags);
+			ret = xml_cluster_status(qs, membership, rgs,
+						 member_name, rg_name,
+						 flags);
 		else
-			txt_cluster_status(qs, membership, rgs, member_name,
-					   rg_name,flags);
+			ret = txt_cluster_status(qs, membership, rgs,
+						 member_name, rg_name,
+						 flags);
 
 		if (membership)
 			cml_free(membership);
